// BIOS-LCD.C - Copyright 1998, Technologic Systems, Inc.
// 	permission is granted for you to use this source code freely.
//
//	This program is a simple example for use with the Technologic
//	Systems line of Embedded PCs. If the platform is a TS-xxxx with
//	an LCD installed, BIOS-LCD will display the time and date on the
//	LCD using the standard printf function. If an LCD is not present, 
//	the program will use the regular display, or the serial console.
//
//	This program will run without modification on a standard
//	desktop PC.
//
// cjd  980910  1.0     Birth
// cjd	990323	1.1	Added '\n' to mesg to display properly on 2x40 LCDs
// cjd	990904	1.2	Fixed to recognize all TS EPC models - not just 2350
// mtj	030121	1.3 Hacked out extraneous code, added time/date display for 3 sec.
//


#include   <stdio.h>
#include   <dos.h>
#include   <stdlib.h>
#include   <conio.h>

#define TS2100 21
#define TS2200 22
#define TS2350 23
#define TS2800 28
#define TS3100 31
#define TS3200 32
#define TS3300 33
#define TS3400 34
#define TS4150 41
#define TS5300 53
#define TS5400 54
#define TS5500 55

const char VER[] = "BIOS-LCD v1.3, (c) 1999-2003 Technologic Systems Inc.";

// Check whether we are running on a TS EPC
int CheckTSBios ()
{
	union REGS regs;

	regs.x.ax = 0xB000;		// TS BIOS info function
	int86 (0x15, &regs, &regs);     // do it.

	if (regs.h.ah == 0 && regs.x.dx == 0x5453) // 'TS' should be returned in DX
		return (regs.h.cl);	// YES - return the model number
	else
		return (0);		// NO this is NOT TS BIOS
} // CheckTSBios

// Make the BIOS call to change the console port. 0 = Video, 1 = COM1, etc
void SetConsolePort (int port)
{
	union REGS regs;

	regs.h.ah = 0xA1;		// Set console redirection function
	regs.x.bx = port;		// Set which port
	int86 (0x15, &regs, &regs);     // do it.
}

int main(void)
{
	struct	time t;
	struct	date d;

	union	REGS regs;

	int	modelnum = 0,
		redirect = 0;

	int count = 5;

	printf ("%s\n", VER);		// print program version

	printf ("Checking platform: ");
	modelnum = CheckTSBios ();
	if (modelnum == 0) {
		printf ("Non-Technologic Computer.\n");
		return 1;
		}
	
	// check for 9500 with DIP #8 on to enable video, if so abort LCD test
	if( inportb( 0x19D ) == 95 ) {

		if( ( inportb( 0x19B ) & 0x80 ) ) {
			printf( "\aTS-9500 video detected, aborting LCD test\n" );
			return 1;
		}
	}

	outportb( 0x7d, 0x10 );		// put into LCD mode

	printf ("Attempting LCD enable... ");
		
	regs.x.ax = 0xB042;			// yes -- attempt LCD enable
	regs.h.bh = 0x01;
	int86 (0x15, &regs, &regs);

	if (regs.h.ah == 0 && regs.h.bh == 1) {
		printf ("Success!\n");
		SetConsolePort (0);		// console redirection must be off for LCD display	
		redirect = 1;			// flag for redirection

		// MAIN DISPLAY LOOP
		while ( count-- ) {
			gettime (&t);
			getdate (&d);

			system ("cls");

			printf ("Testing LCD for 5 seconds:\n");
			printf ("  %2d:%02d:%02d  %d-%02d-%02d ",
				t.ti_hour, t.ti_min, t.ti_sec,
				d.da_year, d.da_mon, d.da_day);

			sleep (1);
		};



	} else {
		redirect = 0;
		printf ("Failed - ");
		switch (regs.h.ah) {
			case 1:
				printf ("no LCD interface present.\n");
				break;
			case 3:
				printf ("is LCD plugged in?\n");
				break;
			default:
				printf ("** Unknown error **\n");
		}
	}


	// If an LCD interface is enabled, disable it now
	if (redirect) {
		printf ("Attempting LCD disable... ");
		system ("cls");
		regs.x.ax = 0xB042;	// turn off LCD
		regs.h.bh = 0x00;
		int86 (0x15, &regs, &regs);

		if (regs.h.ah == 0 && regs.h.bh == 0) {
			printf ("Success!\n");
			SetConsolePort (2);
		} else
			printf ("Failed.\n");
	}

	printf ("\n");

	return 0;
}
